#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.30
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import sh from 'shelljs';
import { input, select, confirm } from '@inquirer/prompts';
import chalk from 'chalk';
import to from 'await-to-done';
import { createArgs } from '@gitmars/utils';
import { getIsGitProject, getGitConfig } from '@gitmars/git';
import { getBuildConfig, getProjectOption, runJenkins } from '@gitmars/build';
import lang from './common/local.mjs';
import { cmdConfig } from './conf/build-mp.mjs';

const { t } = lang;
const { red, yellow } = chalk;
const { args, options } = cmdConfig;
program.name("gitm build-mp").usage(
  "[project] [-e --env [env]] [--api-env [apiEnv]] [-mp --miniprogram [miniprogram]] [-des --description [description]] [-a --app [app]] [-d --data <data>] [-c --confirm]"
).description(t("Launching a jenkins build task of miniprogram"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (project, opt) => {
  const data = JSON.parse(opt.data || "{}");
  let env = opt.env, app = opt.app, _confirm = opt.confirm, build_api_env = opt.build_api_env || data.build_api_env, mini_program = opt.miniprogram || data.mini_program, description = opt.description || data.description;
  if (!project) {
    if (getIsGitProject())
      project = getGitConfig().appName;
    else {
      [, project = ""] = await to(
        input({
          message: t("Enter project name"),
          transformer: (val) => val.trim()
        }).then((val) => val.trim())
      );
    }
  }
  if (!env) {
    [, env] = await to(
      select({
        message: t("Select the environment to build"),
        choices: [
          {
            name: "dev",
            value: "dev",
            description: t("Test environment(alpha)")
          },
          {
            name: "bug",
            value: "bug",
            description: t("Pre-release tag environment(bug)")
          },
          {
            name: "prod",
            value: "prod",
            description: t("Pre-release prod environment(prod)")
          }
        ]
      })
    );
  }
  const [, buildConfig] = await to(getBuildConfig(false));
  const projectOption = await getProjectOption(project, env, buildConfig);
  if (!app) {
    if (!buildConfig) {
      [, app] = await to(
        input({
          message: t("Enter the application to build"),
          transformer: (val) => val.trim()
        }).then((val) => val.trim())
      );
    } else if (projectOption?.apps) {
      [, app] = await to(
        select({
          message: t("Select the application to build"),
          choices: projectOption.apps.map((name) => ({
            name,
            value: name
          }))
        })
      );
    }
  }
  if (!build_api_env) {
    [, build_api_env] = await to(
      select({
        message: t("Select the api environment to build"),
        choices: [
          {
            name: "alpha",
            value: "alpha",
            description: t("Test environment(alpha)")
          },
          {
            name: "tag",
            value: "tag",
            description: t("Pre-release tag environment(bug)")
          },
          {
            name: "release",
            value: "release",
            description: t("Pre-release prod environment(prod)")
          },
          {
            name: "production",
            value: "production",
            description: t("Production environment")
          }
        ]
      })
    );
  }
  if (!mini_program) {
    if (!buildConfig) {
      [, mini_program] = await to(
        input({
          message: t("Generate experiential version of miniprogram"),
          transformer: (val) => val.trim()
        }).then((val) => val.trim())
      );
    } else if (projectOption?.miniprogram) {
      [, mini_program] = await to(
        select({
          message: t("Select the miniprogram to generate experiential version"),
          choices: projectOption.miniprogram
        })
      );
    }
  }
  if (!description) {
    [, description] = await to(
      input({
        message: t("Enter the version description"),
        transformer: (val) => val.trim()
      }).then((val) => val.trim())
    );
  }
  if (!_confirm) {
    let message = `${yellow(t("Please double check the following build parameters"))}
${t(
      "Project Name"
    )}: ${red(project)}
${t("Code Branch")}: ${red(env)}
${t("Build Application")}: ${red(
      app
    )}`;
    message += `
${t("Interface Environment")}: ${red(build_api_env || "production")}`;
    if (mini_program)
      message += `
${t("Experience version pushed to")}: ${red(mini_program || t("Push to templates only"))}`;
    if (description)
      message += `
${t("Version Description")}: ${red(description)}`;
    if ("clean" in data)
      message += `
${t("Clean node modules (use with caution)")}: ${red(data.clean)}`;
    [, _confirm] = await to(confirm({ message }));
  }
  if (!_confirm)
    sh.exit(1);
  else
    runJenkins({
      env,
      project,
      app,
      data: JSON.stringify({ ...data, build_api_env, mini_program, description })
    });
});
program.parse(process.argv);
