#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.30
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import chalk from 'chalk';
import { confirm, checkbox } from '@inquirer/prompts';
import ora from 'ora';
import { waiting } from 'js-cool';
import to from 'await-to-done';
import { getIsGitProject, getConfig, getCurrentBranch, fetch, searchBranches, getIsMergedTargetBranch, getIsBranchOrCommitExist } from '@gitmars/git';
import { echo, createArgs, spawnSync } from '@gitmars/utils';
import lang from './common/local.mjs';
import { cmdConfig } from './conf/cleanbranch.mjs';

const { t } = lang;
const { green, red, yellow } = chalk;
const { args, options } = cmdConfig;
if (!getIsGitProject()) {
  echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const config = getConfig();
function getIsMergedTarget(branch, targets, {
  remote,
  noMerges
}) {
  branch = remote ? "origin/" + branch : branch;
  if (typeof targets === "string")
    targets = [targets];
  for (const target of targets) {
    const isMerged = getIsMergedTargetBranch(branch, target, {
      remote,
      noMerges
    });
    if (!isMerged)
      return false;
  }
  return true;
}
program.name("gitm cleanbranch").usage(
  "[branches...] [-l --list [list]] [-k --key [keyword]] [--exclude [exclude]] [--include [include]] [-t --type [type]] [--target [target]] [-r --remote] [-c --confirm] [-s --strictly]"
).description(t("Clean up merged feature branches"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (branches, opt) => {
  const spinner = ora();
  spinner.color = "green";
  const current = getCurrentBranch();
  async function clean(branch) {
    const removeLocal = getIsBranchOrCommitExist(branch);
    const removeRemote = opt.remote && getIsBranchOrCommitExist(branch, true);
    if (removeLocal || removeRemote) {
      spinner.start(green(t("Deleting: {something}", { something: branch })));
      await waiting(200);
      spinner.succeed(
        green(
          t("Deleted successfully: {something}", {
            something: branch
          })
        )
      );
    }
    if (removeLocal) {
      if (current === branch) {
        spawnSync("git", ["checkout", config.master]);
      }
      spawnSync("git", ["branch", "-D", branch]);
    }
    if (removeRemote) {
      spawnSync("git", ["push", "origin", "--delete", branch]);
    }
  }
  const targets = opt.target ? opt.target.split(",") : [config.develop, config.release];
  fetch();
  if (branches.length === 0) {
    branches = searchBranches({
      remote: opt.remote,
      type: opt.type,
      key: opt.key,
      exclude: opt.exclude,
      include: opt.include
    });
  }
  const _willDeleteBranch = [];
  if (branches.length > 0) {
    if (!opt.list && !opt.confirm) {
      const [, answer] = await to(
        confirm({
          message: t("About to start batch deleting branches, do you want to continue?"),
          default: false
        })
      );
      if (!answer) {
        echo(green(t("exited")));
        process.exit(0);
      }
    }
  } else {
    echo(green(t("No branches were queried.")));
    process.exit(0);
  }
  for (const branch of branches) {
    if ([config.master, config.develop, config.release, config.bugfix, config.support].includes(
      branch
    )) {
      continue;
    }
    spinner.start(
      green(
        t("Start analysis: {something}", {
          something: branch
        })
      )
    );
    const isMerged = getIsMergedTarget(branch, targets, {
      remote: opt.remote,
      noMerges: !opt.strictly
    });
    if (!isMerged) {
      spinner.fail(
        red(
          t("Cannot be deleted: {something}", {
            something: branch
          })
        )
      );
      continue;
    }
    _willDeleteBranch.push(branch);
    await waiting(200);
    spinner.succeed(
      green(
        t("Analysis completed: {something}", {
          something: branch
        })
      )
    );
    if (opt.list) {
      continue;
    }
    await clean(branch);
  }
  spinner.stop();
  if (opt.list) {
    if (_willDeleteBranch.length > 0) {
      console.info("\r");
      const [, selectBranches = []] = await to(
        checkbox({
          message: yellow(
            t(
              "Find {total} branches merged over {branches} branch, please select the branch to clean up",
              {
                branches: targets.join(","),
                total: String(_willDeleteBranch.length)
              }
            )
          ),
          choices: _willDeleteBranch.map((item) => ({
            name: green(item),
            value: item,
            checked: true
          }))
        })
      );
      if (selectBranches.length === 0) {
        echo(yellow(t("No branches were selected and the process has exited.")));
        process.exit(0);
      }
      selectBranches.forEach(async (branch) => {
        await clean(branch);
      });
    } else {
      echo(green(t("Analysis complete, no branches to clean up")));
    }
  } else {
    echo(
      green(
        t("Deletion complete, these branches have been cleaned up") + ": " + _willDeleteBranch.join(" ")
      )
    );
  }
});
program.parse(process.argv);
