#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.30
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { createRequire } from 'node:module';
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import { checkbox } from '@inquirer/prompts';
import to from 'await-to-done';
import { getType } from 'js-cool';
import { isNeedUpgrade, upgradeGitmars, queue } from '@gitmars/core';
import { getIsGitProject, getGitConfig, getConfig, checkGitStatus, getCurrentBranch, searchBranches, getIsMergedTargetBranch, getIsUpdatedInTime } from '@gitmars/git';
import { createArgs } from '@gitmars/utils';
import { getUserInfo } from '@gitmars/api';
import lang from './common/local.mjs';
import { defaults } from './common/global.mjs';
import { cmdConfig } from './conf/combine.mjs';

const { t } = lang;
const require = createRequire(import.meta.url);
const { red, green, yellow } = chalk;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const mergeRequestModule = require.resolve("@gitmars/api");
const { args, options } = cmdConfig;
const { appName } = getGitConfig();
const config = getConfig();
program.name("gitm combine").usage(
  "[type] [name] [-d --dev] [-p --prod] [-b --build [app]] [-a --add] [-m --commit <commit>] [--data <data>] [--description [description]] [--as-feature] [--no-bugfix] [-f --force]"
).description(
  t("Merge bugfix task branch, merge feature development branch, merge support branch")
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (type, name, opt) => {
  const userInfoApi = config.apis?.userInfo?.url || config.api;
  const needUpgrade = await isNeedUpgrade(config.versionControlType);
  needUpgrade && upgradeGitmars();
  const allow = ["bugfix", "feature", "support"];
  const deny = [
    defaults.master,
    defaults.develop,
    defaults.release,
    defaults.bugfix,
    defaults.support
  ];
  const { level, nickname = "" } = userInfoApi ? await getUserInfo() : {};
  const status = !opt.add && opt.commit === "" ? checkGitStatus() : true;
  let _branches = [], pendingBranches = [], isDescriptionCorrect = true;
  if (!opt.dev && !opt.prod) {
    sh.echo(t("Enter the environment to sync to."));
    process.exit(1);
  }
  if (!status)
    process.exit(1);
  if (opt.commit === true) {
    sh.echo(red(t("Enter the message to be submitted")));
    process.exit(1);
  }
  if (config.descriptionValidator) {
    const reg = getType(config.descriptionValidator) === "regexp" ? config.descriptionValidator : new RegExp(config.descriptionValidator);
    isDescriptionCorrect = Boolean(opt.description && reg.test(opt.description));
  }
  if (!type) {
    let _nameArr;
    [type, ..._nameArr] = getCurrentBranch().split("/");
    name = _nameArr.join("/");
    if (!name) {
      deny.includes(type) && sh.echo(
        red(
          t(
            "Hey bro, what is the fuck are you doing by executing this command in the {type} branch?",
            { type }
          )
        )
      );
      process.exit(1);
    }
  }
  if (!allow.includes(type)) {
    sh.echo(red(t("type only allows input") + ": " + JSON.stringify(allow)));
    process.exit(1);
  } else if (type === "feature" && opt.asFeature) {
    sh.echo(t("--as-feature is only used in the bugfix branch."));
    process.exit(1);
  }
  if (!name) {
    _branches = searchBranches({ type });
    if (!_branches.length) {
      sh.echo(red(t("Branch does not exist, please enter it correctly")));
      process.exit(1);
    }
  } else {
    _branches = [type + "/" + name];
  }
  const base = type === "bugfix" ? config.bugfix : config.release;
  const cmd = [];
  const branchesWithInfo = _branches.map((branchName) => {
    const [_type, ..._nameArr] = branchName.split("/");
    const _name = _nameArr.join("/");
    const isNeedCombineDevelop = !getIsMergedTargetBranch(branchName, config.develop, {
      remote: true
    });
    const isNeedCombineBase = !getIsMergedTargetBranch(branchName, base, {
      remote: true
    });
    const isNeedCombineRelease = !getIsMergedTargetBranch(branchName, config.release, {
      remote: true
    });
    const isNeedCombineBugfix = !getIsMergedTargetBranch(branchName, config.bugfix, {
      remote: true
    });
    return {
      branchName,
      type: _type,
      name: _name,
      isNeedCombineDevelop,
      isNeedCombineBase,
      isNeedCombineRelease,
      isNeedCombineBugfix
    };
  });
  if (branchesWithInfo.length === 1)
    pendingBranches = branchesWithInfo;
  else {
    [, pendingBranches = []] = await to(
      checkbox({
        message: t("Select branch for batch processing"),
        choices: branchesWithInfo.map((item) => {
          const _merged = [];
          if (!item.isNeedCombineDevelop)
            _merged.push(config.develop);
          if (!item.isNeedCombineRelease)
            _merged.push(config.release);
          if (!item.isNeedCombineBugfix)
            _merged.push(config.bugfix);
          return {
            name: t("{source}{info}", {
              source: item.branchName,
              info: _merged.length ? green(
                " (" + t(`Merged branch: {info}`, {
                  info: _merged.join("/")
                }) + ")"
              ) : ""
            }),
            value: item,
            checked: false
          };
        })
      })
    );
  }
  if (!pendingBranches.length) {
    sh.echo(t("No pending branches, program exits"));
    process.exit(0);
  }
  if (opt.add) {
    cmd.push("git add .");
  }
  if (opt.commit) {
    cmd.push(`git commit -m "${opt.commit}"`);
  }
  for (const {
    branchName,
    type: type2,
    isNeedCombineDevelop,
    isNeedCombineBase,
    isNeedCombineRelease,
    isNeedCombineBugfix
  } of pendingBranches) {
    if (!getIsUpdatedInTime({ lastet: "7d", limit: 1e3, branch: base })) {
      sh.echo(
        yellow(
          t(
            "The {source} branch has not updated its upstream branch code in over a week, please sync it at least once a week, execute: gitm update (-f)",
            {
              source: branchName
            }
          )
        )
      );
    }
    if (opt.dev) {
      if (isNeedCombineDevelop || opt.force)
        cmd.push(
          "git fetch",
          `git checkout ${config.develop}`,
          "git pull",
          {
            cmd: `git merge --no-ff ${branchName}`,
            config: {
              again: false,
              success: t("Merge {source} into {target} successfully", {
                source: branchName,
                target: config.develop
              }),
              fail: t(
                "An error occurred merging {source} to {target}, Please follow the instructions",
                {
                  source: branchName,
                  target: config.develop
                }
              )
            }
          },
          {
            cmd: "git push",
            config: {
              again: true,
              success: t("Successful Pushed"),
              fail: t("Push failed, please follow the prompts")
            }
          },
          `git checkout ${branchName}`
        );
      else
        cmd.push({
          message: t("{source} has been merged with {target}", {
            source: branchName,
            target: config.develop
          })
        });
    }
    if (opt.prod) {
      if (!opt.dev && isNeedCombineDevelop) {
        sh.echo(
          yellow(
            t(
              "If your branch has not been merged into {target}, please merge it into the {target} branch first",
              { target: config.develop }
            )
          )
        );
        process.exit(1);
      }
      if (!opt.noBugfix && !opt.asFeature) {
        const weekday = (/* @__PURE__ */ new Date()).getDay();
        if ([0, 4, 5].includes(weekday) && base === config.bugfix) {
          sh.echo(
            yellow(
              t("Use the {base} branch for posting on Thursday, Friday, and Sunday", {
                base: config.release
              })
            )
          );
        }
        if (!level || level < 3) {
          if (isNeedCombineBase || opt.force)
            cmd.push(
              "git fetch",
              `git checkout ${base}`,
              "git pull",
              {
                cmd: `git merge --no-ff ${branchName}`,
                config: {
                  again: false,
                  success: t("Merge {source} into {target} successfully", {
                    source: branchName,
                    target: base
                  }),
                  fail: t(
                    "An error occurred merging {source} to {target}, Please follow the instructions",
                    {
                      source: branchName,
                      target: base
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: t("Successful Pushed"),
                  fail: t("Push failed, please follow the prompts")
                }
              },
              `git checkout ${branchName}`
            );
          else
            cmd.push({
              message: t("{source} has been merged with {target}", {
                source: branchName,
                target: base
              })
            });
        } else {
          if (!isDescriptionCorrect) {
            sh.echo(
              red(
                t(
                  "The description of the reason for submission does not meet the specification"
                )
              )
            );
            process.exit(1);
          }
          cmd.push(
            {
              cmd: `git push --set-upstream origin ${branchName}`,
              config: {
                again: true,
                success: t("Push remote and associate remote branch successfully"),
                fail: t("Push remote failed, please follow the prompts")
              }
            },
            {
              cmd: {
                module: mergeRequestModule,
                entry: "createMergeRequest",
                options: {
                  source_branch: branchName,
                  target_branch: base,
                  description: opt.description
                }
              },
              config: {
                again: true,
                success: t("Successfully created merge request"),
                fail: t(
                  "There was an error creating the merge request, please follow the instructions"
                )
              }
            },
            [
              "gitm",
              "postmsg",
              `"${t(
                "{nickname} submitted a merge request for {source} branch to {target} branch in {app} project",
                {
                  nickname,
                  app: appName,
                  source: branchName,
                  target: base
                }
              )}"`
            ]
          );
        }
      }
      if (type2 === "bugfix" && opt.asFeature) {
        if (!level || level < 3) {
          if (isNeedCombineRelease || opt.force)
            cmd.push(
              "git fetch",
              `git checkout ${config.release}`,
              "git pull",
              {
                cmd: `git merge --no-ff ${branchName}`,
                config: {
                  again: false,
                  success: t("Merge {source} into {target} successfully", {
                    source: branchName,
                    target: config.release
                  }),
                  fail: t(
                    "An error occurred merging {source} to {target}, Please follow the instructions",
                    {
                      source: branchName,
                      target: config.release
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: t("Successful Pushed"),
                  fail: t("Push failed, please follow the prompts")
                }
              },
              `git checkout ${branchName}`
            );
          else
            cmd.push({
              message: t("{source} has been merged with {target}", {
                source: branchName,
                target: config.release
              })
            });
        } else {
          if (!isDescriptionCorrect) {
            sh.echo(
              red(
                t(
                  "The description of the reason for submission does not meet the specification"
                )
              )
            );
            process.exit(1);
          }
          cmd.push(
            {
              cmd: `git push --set-upstream origin ${branchName}`,
              config: {
                again: true,
                success: t("Push remote and associate remote branch successfully"),
                fail: t("Push remote failed, please follow the prompts")
              }
            },
            {
              cmd: {
                module: mergeRequestModule,
                entry: "createMergeRequest",
                options: {
                  source_branch: branchName,
                  target_branch: config.release,
                  description: opt.description
                }
              },
              config: {
                again: true,
                success: t("Successfully created merge request"),
                fail: t(
                  "There was an error creating the merge request, please follow the instructions"
                )
              }
            },
            [
              "gitm",
              "postmsg",
              `"${t(
                "{nickname} submitted a merge request for {source} branch to {target} branch in {app} project",
                {
                  nickname,
                  app: appName,
                  source: branchName,
                  target: config.release
                }
              )}"`
            ]
          );
        }
      }
      if (type2 === "support" && opt.noBugfix) {
        if (!level || level < 3) {
          if (isNeedCombineBugfix || opt.force)
            cmd.push(
              "git fetch",
              `git checkout ${config.bugfix}`,
              "git pull",
              {
                cmd: `git merge --no-ff ${branchName}`,
                config: {
                  again: false,
                  success: t("Merge {source} into {target} successfully", {
                    source: branchName,
                    target: config.bugfix
                  }),
                  fail: t(
                    "An error occurred merging {source} to {target}, Please follow the instructions",
                    {
                      source: branchName,
                      target: config.bugfix
                    }
                  )
                }
              },
              {
                cmd: "git push",
                config: {
                  again: true,
                  success: t("Successful Pushed"),
                  fail: t("Push failed, please follow the prompts")
                }
              },
              `git checkout ${branchName}`
            );
          else
            cmd.push({
              message: t("{source} has been merged with {target}", {
                source: branchName,
                target: config.bugfix
              })
            });
        } else {
          if (!isDescriptionCorrect) {
            sh.echo(
              red(
                t(
                  "The description of the reason for submission does not meet the specification"
                )
              )
            );
            process.exit(1);
          }
          cmd.push(
            {
              cmd: `git push --set-upstream origin ${branchName}`,
              config: {
                again: true,
                success: t("Push remote and associate remote branch successfully"),
                fail: t("Push remote failed, please follow the prompts")
              }
            },
            {
              cmd: {
                module: mergeRequestModule,
                entry: "createMergeRequest",
                options: {
                  source_branch: branchName,
                  target_branch: config.bugfix,
                  description: opt.description
                }
              },
              config: {
                again: true,
                success: t("Successfully created merge request"),
                fail: t(
                  "There was an error creating the merge request, please follow the instructions"
                )
              }
            },
            [
              "gitm",
              "postmsg",
              `"${t(
                "{nickname} submitted a merge request for {source} branch to {target} branch in {app} project",
                {
                  nickname,
                  app: appName,
                  source: branchName,
                  target: config.bugfix
                }
              )}"`
            ]
          );
        }
      }
    }
  }
  if (opt.build) {
    if (opt.dev)
      cmd.push({
        cmd: `gitm build ${appName} --confirm --env dev --app ${opt.build === true ? "all" : opt.build} ${opt.data ? " --data " + opt.data : ""}`,
        config: {
          stdio: "inherit",
          again: false,
          success: t("Pulling up the build was successful"),
          fail: t("Failed to pull up the build")
        }
      });
    else if (opt.prod) {
      if (!level || level < 3) {
        if (type === "bugfix") {
          cmd.push({
            cmd: `gitm build ${appName} --confirm --env bug --app ${opt.build === true ? "all" : opt.build} ${opt.data ? " --data " + opt.data : ""}`,
            config: {
              stdio: "inherit",
              again: false,
              success: t("Pulling up the build was successful"),
              fail: t("Failed to pull up the build")
            }
          });
        }
        if (type === "support" && opt.noBugfix) {
          cmd.push({
            cmd: `gitm build ${appName} --confirm --env bug --app ${opt.build === true ? "all" : opt.build} ${opt.data ? " --data " + opt.data : ""}`,
            config: {
              stdio: "inherit",
              again: false,
              success: t("Pulling up the build was successful"),
              fail: t("Failed to pull up the build")
            }
          });
        }
      } else {
        sh.echo(
          yellow(
            t(
              "This process will not automatically execute the build process, please wait for the administrator to review the code and execute it: gitm build {appName} -e prod -a {app} {data}",
              {
                appName,
                app: opt.build === true ? "all" : opt.build,
                data: opt.data ? " --data " + opt.data : ""
              }
            )
          )
        );
      }
    }
  }
  queue(cmd);
});
program.parse(process.argv);
